(* Scene description in smoke. Likely to change. This is meant to be the    *)
(* complex interface supporting data sharing (e.g. the same fill type with  *)
(* multiple objects). A simpler interface could be built on top of this     *)
(* which used values for simplicity?)                                       *)

(* Also: The nicest set of datatypes depends on how they are used. Will have*)
(* to keep changing these to fit.                                           *)

type rgba = { red: float; green: float; blue: float; alpha: float }

type vector = float * float

type fill = 
  |  FlatFill of rgba 
  |  GradientFill of rgba * vector * rgba * vector
(*                   <---one----->   <---two------> *)


type join_style = 
  |  JoinMitred
  |  JoinBevelled
  |  JoinRounded

type endcap_style = 
  |  EndcapNone
  |  EndcapButt
  |  EndcapRounded

type line_style = { line_fill: fill Index.handle;
		    line_width: float;
		    line_join_style: join_style;
		    line_endcap_style: endcap_style }

type segment = 
  |  Line of vector
  |  Bezier of vector * vector * vector

type contour = 
  |  Open of vector * segment list
  |  Closed of segment list

type canvasObject = 
  |  ContourObject of fill Index.handle list * line_style Index.handle list 
        * contour list
  |  TextObject of  fill Index.handle list * line_style Index.handle list 
        * vector * float * string 

type scene = 
  { fill_types: fill Index.index;
    line_styles: line_style Index.index;
    objects: canvasObject list ref}

(* ------------------------------------------------------------------------ *)
(* Functions to create smoke scenes                                         *)

let blank_scene () : scene = 
  { fill_types = Index.create();
    line_styles = Index.create();
    objects = ref [] }

let add_fill_type scene f = 
  match (Index.find_key_for_val scene.fill_types f) with
  | Some key -> key
  | None -> Index.insert scene.fill_types f

let add_line_style scene ls = 
  match (Index.find_key_for_val scene.line_styles ls) with
  | Some key -> key
  | None -> Index.insert scene.line_styles ls

let add_object scene o = 
  scene.objects := o:: (!(scene.objects))
