(* ------------------------------------------------------------------------ *)
(* Code to output in the file format Smoke uses                             *)

open List

open Smoke

let colour rgba = 
  (string_of_float rgba.red) ^ " " ^ 
  (string_of_float rgba.green) ^ " " ^ 
  (string_of_float rgba.blue) ^ " " ^ 
  (string_of_float rgba.alpha) ^ "\n"

let vec (x, y) = 
  (string_of_float x) ^ " " ^ (string_of_float y)

let write_fill writer = function
  | FlatFill rgba -> 
      writer "FlatFill\n";
      writer (colour rgba)
  | GradientFill (rgba_1, vec_1, rgba_2, vec_2) ->
      writer "GradientFill\n";
      writer (colour rgba_1);
      writer ((vec vec_1) ^ "\n");
      writer (colour rgba_2);
      writer ((vec vec_2) ^ "\n")

let write_fill_types writer scene  =
  let all_handles = Index.all_keys scene.fill_types in
  let all_fill_types = map (fun h -> Index.find scene.fill_types h) 
      all_handles in
  writer "FillTypes\n";
  iter (write_fill writer) all_fill_types;
  writer "\n"

let write_join_style writer = function
  | JoinMitred -> writer "mitred\n"
  | JoinBevelled -> writer "bevelled\n"
  | JoinRounded -> writer "rounded\n"

let write_endcap_style writer = function
  | EndcapNone -> writer "none\n"
  | EndcapButt -> writer "butt\n"
  | EndcapRounded -> writer "rounded\n"

let write_line_style writer style = 
  (*writer ((match style.line_fill with
           | None -> ""
	   | Some h -> string_of_int (Index.int_of_handle h)) ^ "\n");*)
  writer ((string_of_int (Index.int_of_handle (style.line_fill))) ^ "\n");
  writer ((string_of_float (style.line_width)) ^ "\n");
  write_join_style writer (style.line_join_style);
  write_endcap_style writer (style.line_endcap_style)

let write_line_styles writer scene = 
  let all_handles = Index.all_keys scene.line_styles in
  let all_line_styles = map (fun h -> Index.find scene.line_styles h) 
      all_handles in
  writer "LineStyles\n";
  iter (write_line_style writer) all_line_styles;
  writer "\n"

let write_segment writer = function
  | Line v -> writer ("Line " ^ (vec v) ^ "\n")
  | Bezier (a, b, c) -> writer ("Bezier " ^ (vec a) ^ " " ^ (vec b) ^ 
				" " ^ (vec c) ^ "\n")

let write_contour writer = function
  | Open (v, segments) ->
      writer "Contour\n";
      writer ("Open " ^ (vec v) ^ "\n");
      iter (write_segment writer) segments;
      writer "\n";
  | Closed segments ->
      writer "Contour\n";
      writer "\n";
      iter (write_segment writer) segments;
      writer "\n"


let write_canvasObject writer = function
  | ContourObject (fills, line_styles, contours) ->
      writer "ContourObject\n";
      writer ("filltypes " ^ 
	      (String.concat " " 
		 (map (fun x -> string_of_int (Index.int_of_handle x)) fills))
	      ^ "\n");
      writer ("linestyles " ^ 
	      (String.concat " " 
		 (map (fun x -> string_of_int (Index.int_of_handle x)) 
		    line_styles)) ^ "\n");
      iter (write_contour writer) contours;
      writer "\n";
  | _ -> ()

let write_scene writer scene = 
  write_fill_types writer scene;
  write_line_styles writer scene;
  writer "Scene \n"; 
  iter (write_canvasObject writer) !(scene.objects)
  
let write scene = write_scene print_string scene
  

      
