#!/usr/bin/env perl

# Grab the webpage http://www.w3.org/TR/SVG/types.html#ColorKeywords
# and extract the "color keywords"

use strict;
  
use File::Temp qw/ tempfile tempdir /;

my $url = "http://www.w3.org/TR/SVG/types.html#ColorKeywords";
my $wget = "wget";

my $tempdir = tempdir();
chdir ($tempdir);
system ("$wget $url");

die("Failed to wget URL: $url ") if (!-e "types.html");
open(FILE, "types.html") or die("Failed to open html file $!");

my $program = $0;
my $date = `date`;
format HEADER_COMMENT = 
(* Autogenerated file -- DO NOT MODIFY BY HAND!                              *)
(* File was generated by @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<                      *)
                         $program
(*                    on @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<                      *)
                         $date
(*                  from @<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<      *)
                         $url

let string_mapping  = Hashtbl.create 100 (* maps strings onto SVG colours    *)

open Svgtype

.
    $~ = "HEADER_COMMENT";
write;

my ($colour_name, $red, $green, $blue);

format COLOUR_RECORD = 
let @<<<<<<<<<<<<<<<<<<<<<<<<<< = { red = @<<<; green = @<<<; blue = @<<< }
    $colour_name,                         $red,        $green,       $blue
.

    $~ = "COLOUR_RECORD";

# Locate the anchor point
while ($_ = <FILE>){
    last if (/name=\"ColorKeywords\"/);
}

while ($_ = <FILE>){
    my $buffer = $_;

# This occurs just after the table in the file
    last if ($buffer =~ /BasicDOMInterfaces/);

    while ($buffer =~ /prop-value\"\>(.+?)\<\/span\>(.+)/){
	$colour_name = $1;
	$buffer = $2;
	if ($buffer =~ /color-keyword-value\"\>rgb\(\s*(\d+)\s*,\s*(\d+)\s*,\s*(\d+)\s*\)(.+)/){
	    $red = $1; $green = $2; $blue = $3;
	    write;
	    print "let _ = Hashtbl.add string_mapping \"$colour_name\" $colour_name\n\n";
	    $buffer = $4;
	}

    }


}

print <<EOT;
(* lookup: string -> colour *)
let lookup = Hashtbl.find string_mapping

EOT
close(FILE);

unlink("$tempdir/types.html");
unlink($tempdir);

