(* Main path-data definition. Has the full "zoo" of path-data elements.     *)

(* Need to write a simplifier to remove all the "Shorthand" stuff.          *)

type coord = { x: float; y: float }

type absrel =
  | Absolute
  | Relative

type arc_data = { radii: coord;
		  xaxisrot: float;
		  largearc: bool;
		  sweep: bool;
		  curveto: coord }

type path = 
  | MoveTo of absrel * coord
  | ClosePath of absrel
  | LineTo of absrel * coord
  | HorizLineTo of absrel * float
  | VertLineTo of absrel * float
  | CurveTo of absrel * (coord * coord * coord)
  | ShorthandCurveTo of absrel * (coord * coord)
  | QuadCurveTo of absrel * (coord * coord)
  | ShorthandQuadCurveTo of absrel * coord
  | Arc of absrel * arc_data


(* Pretty printing code                                                     *)

let pp_absrel = function
  | Absolute -> "Absolute"
  | Relative -> "Relative"

let pp_coord { x=x; y=y } = "(" ^ (string_of_float x) ^ "," ^ 
                                  (string_of_float y) ^ ")"

let pp_arcdata data = "radii = " ^ (pp_coord data.radii) ^ " " ^
                      "xaxisrot = " ^ (string_of_float data.xaxisrot) ^ " " ^
                      "largearc = " ^ (string_of_bool data.largearc) ^ " " ^
                      "sweep = " ^ (string_of_bool data.sweep) ^ " " ^
                      "curveto = " ^ (pp_coord data.curveto)

let pp_path = function
  | MoveTo (a, xy) -> "MoveTo " ^ (pp_absrel a) ^ " " ^ (pp_coord xy)
  | ClosePath a -> "ClosePath " ^ (pp_absrel a)
  | LineTo (a, xy) -> "LineTo " ^ (pp_absrel a) ^ " " ^ (pp_coord xy)
  | HorizLineTo (a, x) -> "HorizLineTo " ^ (pp_absrel a) ^ " " ^ 
      (string_of_float x)
  | VertLineTo (a, y) -> "VertLineTo " ^ (pp_absrel a) ^ " " ^ 
      (string_of_float y)
  | CurveTo (a, (p, q, r)) -> "CurveTo " ^ (pp_absrel a) ^ " " ^
      (pp_coord p) ^ " " ^ (pp_coord q) ^ " " ^ (pp_coord r)
  | ShorthandCurveTo (a, (p, q)) -> "ShorthandCurveTo " ^ (pp_absrel a) ^ " " ^
      (pp_coord p) ^ " " ^ (pp_coord q)
  | QuadCurveTo (a, (p, q)) -> "QuadCurveTo " ^ (pp_absrel a) ^ " " ^
      (pp_coord p) ^ " " ^ (pp_coord q)
  | ShorthandQuadCurveTo (a, p) -> "ShorthandQuadCurveTo " ^ 
      (pp_absrel a) ^ " " ^(pp_coord p)
  | Arc (a, data) -> "Arc " ^ (pp_absrel a) ^ " " ^ (pp_arcdata data)
