
%{
open Path (* for the definition of the path-data elements *)

(* let parse_error msg = raise (Parse_error_at (Parsing.symbol_end())) *)

exception ParseError of string

let fst1 x = x
let fst2 (x, _) = x

let to_coord x y = { x=x; y=y }
let to_float = float_of_string
let to_flag = function
 | 0.0 -> false (* danger, danger. should not use floats for this *)
 | 1.0 -> true
 | _ -> raise (ParseError "Arc flag value must be 0 or 1")


let ty = ref Relative

(* The grammar reads a path in as a list of tuples of coordinates and other
   data. The context in the grammar is then used to call the following helper
   functions which create path segments from the lists of tuples. *)

let rec convert_arcs a y = match y with
  [] -> []
| (r, x, l, s, e)::t -> Arc(a, { radii=r;
			         xaxisrot = x;
			         largearc = l;
			         sweep = s;
			         curveto = e })::(convert_arcs a t);;

let rec convert_short_qcurves a y = match y with
  [] -> []
| cs::t -> ShorthandQuadCurveTo(a, cs)::(convert_short_qcurves a t);;

let rec convert_qcurves a y = match y with
  [] -> []
| cs::t -> QuadCurveTo(a, cs)::(convert_qcurves a t);;

let rec convert_short_curves a y = match y with
  [] -> []
| cs::t -> ShorthandCurveTo(a, cs)::(convert_short_curves a t);;

let rec convert_curves a y = match y with
  [] -> []
| cs::t -> CurveTo(a, cs)::(convert_curves a t);;

let rec convert_hlines a y = match y with
  [] -> []
| cs::t -> HorizLineTo(a, cs)::(convert_hlines a t);;

let rec convert_vlines a y = match y with
  [] -> []
| cs::t -> VertLineTo(a, cs)::(convert_vlines a t);;

let rec convert_lines a y = match y with
  [] -> []
| cs::t -> LineTo(a, cs)::(convert_lines a t);;

let convert_move a y = match y with
  [] -> []
| cs::t -> MoveTo(a, cs)::(convert_lines a t);;

%}

%token <float> NUMBER
%token <Path.absrel> MOVE
%token <Path.absrel> CLOSE
%token <Path.absrel> LINE
%token <Path.absrel> HORIZLINE
%token <Path.absrel> VERTLINE
%token <Path.absrel> CURVE
%token <Path.absrel> SHORTHANDCURVE
%token <Path.absrel> QUADCURVE
%token <Path.absrel> SHORTHANDQUADCURVE
%token <Path.absrel> ARC
%token EOF

%start path             /* the entry point */
%type <Path.path list> path
        
%%

flag:
|  NUMBER                     { to_flag $1 }

coord:
|  NUMBER NUMBER              { to_coord $1 $2 }

coord2:
|  coord coord                { ($1, $2) }

coord3:
|  coord coord coord          { ($1, $2, $3) }

coord_list:
 | coord                      { [ $1 ] }
 | coord coord_list           {  ($1)::$2 }

coord2_list:
 | coord2                     { [ $1 ] }
 | coord2 coord2_list         {  ($1)::$2 }

coord3_list:
 | coord3                     { [ $1 ] }
 | coord3 coord3_list         {  ($1)::$2 }

number_list:
 | NUMBER                     { [ $1 ] }
 | NUMBER number_list         {  ($1)::$2 }

arc_arg:
 | coord NUMBER flag flag coord
                              { ($1, $2, $3, $4, $5) }
arc_args:
 | arc_arg                    { [ $1 ] }
 | arc_arg arc_args           {  ($1)::$2 }


element:
 | CLOSE                      { [ ClosePath ($1) ] }

 | MOVE coord_list            { convert_move $1 $2 }
 | LINE coord_list            { convert_lines $1 $2 }
 | HORIZLINE number_list      { convert_hlines $1 $2 }
 | VERTLINE number_list       { convert_vlines $1 $2 }
 | CURVE coord3_list          { convert_curves $1 $2 }
 | SHORTHANDCURVE coord2_list { convert_short_curves $1 $2 }
 | QUADCURVE coord2_list      { convert_qcurves $1 $2 }
 | SHORTHANDQUADCURVE coord_list
                              { convert_short_qcurves $1 $2 }
 | ARC arc_args               { convert_arcs $1 $2 }

path:
 | EOF                        { [] }
 | element path               { $1 @ $2 }
