
(* ML datatypes corresponding to the structures found in the SVG document   *)
(* Incrementally build this up in sync with xmlread.                        *)

type colour = {red: float; green: float; blue: float; alpha: float}

type style_option = 
  | Fill of colour
  | Stroke of colour
  | StrokeWidth of float
(* ... *)

type group_type = { group_style: style_option list;
		    group_objects: obj list; }

and path_type = { path_data: Path.path list }

and obj = 
  | Group of group_type
  | Path of  path_type
  | Svg of group_type (* I think *)

(* ------------------------------------------------------------------------ *)

let pp_colour rgba = "{ red = " ^ (string_of_float rgba.red) ^ 
                     "  green = " ^ (string_of_float rgba.green) ^
                     "  blue = " ^ (string_of_float rgba.blue) ^ " }"

let pp_style_option = function
  | Fill rgba -> "Fill " ^ (pp_colour rgba)
  | Stroke rgba -> "Stroke " ^ (pp_colour rgba)
  | StrokeWidth x -> "StrokeWidth " ^ (string_of_float x)

let pp_style_option_list x = 
  "[" ^ (String.concat "," (List.map pp_style_option x)) ^ "]"

let rec pp_group_style gt = 
  "style = " ^ (pp_style_option_list gt.group_style) ^ 
  " objects = { " ^ 
  (String.concat "\n" (List.map pp_obj gt.group_objects)) ^
  "\n}"

and pp_obj = function
  | Group gt -> "Group " ^ (pp_group_style gt)
  | Svg gt -> "Svg " ^ (pp_group_style gt)
  | Path pt ->
      "Path data = " ^ 
      (String.concat "\n" 
	 (List.map Path.pp_path pt.path_data)^"\n")
