(* Implementation of index.mli interface                                    *)

type 'a handle = int (* shouldnt expose this really *)

(* An index is a reference to the next key paired with a key -> value       *)
(* hanshtable.                                                              *)

type 'a index = ( 'a handle ref * ('a handle, 'a) Hashtbl.t )

let create () = ( ref 0, Hashtbl.create 10 )

let insert (key, hash) item =
  let this_key = !key in
  Hashtbl.add hash this_key item;
  key := !key + 1;
  this_key

let find (_, hash) key = Hashtbl.find hash key

let rec count from upto = 
  if (from > upto) then []
  else if (from = upto) then [ from ]
  else from::(count (from+1) upto)

let all_keys (key, hash) = 
  count 0 (!key - 1)

let int_of_handle h = h


let find_key_for_val (key, hash) v = 
  let rec check_key k = 
    if (k >= !key) then None
    else let this_val = find (key, hash) k in
    if (v = this_val) then Some k
    else check_key (k+1)
  in check_key 0
