(* Interface for a hashtable-like structure mapping ('a handle) values onto *)
(* values of type ('a).                                                     *)

(* Example, if you want to create an index of type "rgb" then you can do    *)
(* something like the following:                                            *)
(*   let (table : rgb index) = create()                                     *)
(*   let h = insert table rgb_value                                         *)
(*     (h is now of type (rgb handle))                                      *)
(*   let rgb_value = find table h                                           *)

type 'a handle
type 'a index

val create : unit -> 'a index
val insert : 'a index -> 'a -> 'a handle
val find : 'a index -> 'a handle -> 'a 

val all_keys : 'a index -> 'a handle list
val find_key_for_val : 'a index -> 'a -> 'a handle option

(* Slight hack... to serialize this to disk I expose the implementation.    *)
(* Really should provide a stream serialization/deserialization pair        *)
val int_of_handle : 'a handle -> int

