/** file: autoptr.cc
 */
/*
    Smoke Vector Graphics engine
    Copyright (C) 1998-2003  Jon Harrop

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/



template<class T>
AutoPtr<T>::AutoPtr(T* p) : ptr(p)
{
}

template<class T>
AutoPtr<T>::AutoPtr(const AutoPtr& a) : ptr(a.release())
{
}

template<class T>
template<class T1>
AutoPtr<T>::AutoPtr(const AutoPtr<T1> &a) : ptr(a.release())
{
}

template<class T>
T *
AutoPtr<T>::operator()(T *)
{
  return ptr;
}

template<class T>
AutoPtr<T> &
AutoPtr<T>::operator= (AutoPtr &a)
{
  if (&a != this)
    {
      delete ptr;
      ptr = a.release();
    }

  return *this;
}

template<class T>
template<class T1>
AutoPtr<T> &
AutoPtr<T>::operator= (AutoPtr<T1> &a)
{
  if (&a != this)
    {
      delete ptr;
      ptr = a.release();
    }

  return *this;
}

template<class T>
AutoPtr<T>::~AutoPtr()
{
  delete ptr;
  ptr = 0;
}

template<class T>
T& AutoPtr<T>::operator*() const
{
  return *ptr;
}

template<class T>
T* AutoPtr<T>::operator->() const
{
  return ptr;
}

template<class T>
AutoPtr<T>::operator T *()
{
  return ptr;
}

template<class T>
T* AutoPtr<T>::release() const
{
  T* tmp = ptr;
  ptr = 0;
  return tmp;
}

template<class T>
void AutoPtr<T>::reset(T * p)
{
  delete ptr;
  ptr = p;
}
