/** file: bound.cc
 */
/*
    Smoke Vector Graphics engine
    Copyright (C) 1998-2003  Jon Harrop

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "bound.h"

#include <list>



// Bound

Bound::Bound()
{
}

Bound::Bound(Vec nl,
	     Vec nu)
  : l(nl),
    u(nu)
{
}

Bound::Bound(Bound a,
	     Bound b)
{
  for (int i=0; i<2; i++)
    {
      l(i) = std::min(a.l(i), b.l(i));
      u(i) = std::max(a.u(i), b.u(i));
    }
}

bool
Bound::covers(Bound b) const
{
  for (int i=0; i<2; i++)
    if (l(i)>b.l(i) || u(i)<b.u(i))
      return false;
  return true;
}

bool
Bound::distinctFrom(Bound b) const
{
  for (int i=0; i<2; i++)
    if (l(i)>b.u(i) || u(i)<b.l(i))
      return true;
  return false;
}

bool
Bound::contains(Vec r) const
{
  for (int i=0; i<2; i++)
    if (l(i)>r(i) || u(i)<r(i))
      return false;
  return true;
}



Bound
intersection(const Bound &a,
	     const Bound &b)
{
  Bound c;

  for (int i=0; i<2; i++)
    {
      c.l(i) = std::max(a.l(i), b.l(i));
      c.u(i) = std::min(a.u(i), b.u(i));
    }

  // If there was no intersection then return a null bound
  for (int i=0; i<2; i++)
    if (c.l(i) > c.u(i))
      return Bound(Vec(0, 0), Vec(0, 0));

  return c;
}



Bound
operator*(Mat m,
	  Bound b)
{
  // Calculate the four corners of the transformed bound
  TinyVector<Vec, 4> p(m*b.l,
		       m*Vec(b.l(0), b.u(1)),
		       m*Vec(b.u(0), b.l(1)),
		       m*b.u);

  // Find the bounding box of these vertices
  Vec l=p(0), u=p(0);
  for (int i=1; i<4; i++)
    {
      Vec &t=p(i);
      for (int j=0; j<2; j++)
	{
	  if (t(j) < l(j)) l(j)=t(j);
	  if (t(j) > u(j)) u(j)=t(j);
	}
    }

  return Bound(l, u);
}
