/** file: bound.h
 */
/*
    Smoke Vector Graphics engine
    Copyright (C) 1998-2003  Jon Harrop

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

class Bound;

#ifndef BOUND_H
#define BOUND_H

#include "num.h"
#include "vecmat.h"



// OPTIM: Bounding areas could be improved by using arbitrary convex polygons
// instead of axis-aligned rectangles.

/** Axis aligned rectangle. */
class Bound
{
 public:
  Bound();

  /** Contruct a bound with the given corners. */
  Bound(Vec nl,
	Vec nu);

  /** Contruct a bound which covers both of the given bounds. */
  Bound(Bound a,
	Bound b);

  Vec
    l, /*<- Bottom left corner. */
    u; /*<- Top right corner. */

  /** Calculate if this bound completely covers the given bound. */
  bool
    covers(Bound b) const;

  /** Calculate if this bound does not overlap with the given bound. */
  bool
    distinctFrom(Bound b) const;

  /** Calculate if this bound contains the given point. */
  bool
    contains(Vec r) const;
};



/** Calculate the bound that is the intersection of the two given bounds. */
Bound
intersection(const Bound &a,
	     const Bound &b);



Bound
operator*(Mat m,
	  Bound b);



#endif /* BOUND_H */
