/** file: caches.cc
 */
/*
    Smoke Vector Graphics engine
    Copyright (C) 1998-2003  Jon Harrop

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "caches.h"



// CachedTesselation

CachedTesselation::CachedTesselation()
{
}

bool
CachedTesselation::insert(GLenum winding_rule,
			  GLDisplayList *dl)
{
  // Look for an existing cache entry
  Interiors::const_iterator it=interiors.find(winding_rule);

  // If one was found then return this display list unused
  if (it != interiors.end()) return false;

  // Otherwise insert a new cache entry and return this display list as used
  interiors[winding_rule] = dl;
  return true;
}

bool
CachedTesselation::insert(const StrokeStyle &style,
			  GLDisplayList *dl)
{
  Strokes::const_iterator it=strokes.find(style);
  if (it != strokes.end()) return false;
  strokes[style] = dl;
  return true;
}

bool
CachedTesselation::render(GLenum winding_rule) const
{
  // Look for an existing cache entry
  Interiors::const_iterator it=interiors.find(winding_rule);

  // If one was not found then return, failing to render
  if (it == interiors.end()) return false;

  // Otherwise fetch and execute the corresponding display list and return
  // success
  GLDisplayList *dl=it->second;
  dl->exec();
  return true;
}

bool
CachedTesselation::render(const StrokeStyle &style) const
{
  Strokes::const_iterator it=strokes.find(style);
  if (it == strokes.end()) return false;
  GLDisplayList *dl=it->second;
  dl->exec();
  return true;
}

void
CachedTesselation::clearInteriors()
{
  interiors.clear();
}

void
CachedTesselation::clearStrokes()
{
  strokes.clear();
}

void
CachedTesselation::clear()
{
  clearInteriors();
  clearStrokes();
}
