/** file: caches.h
 */
/*
    Smoke Vector Graphics engine
    Copyright (C) 1998-2003  Jon Harrop

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

class CachedTesselation;

#ifndef CACHES_H
#define CACHES_H

#include "store.h"
#include "geometry.h"
#include "gldisplaylist.h"
#include "stroke.h"
#include "scene.h"

#include <map>
#include <GL/glu.h>



/** Cached interior and stroke tesselations at the same deviation. */
class CachedTesselation
{
 private:
  typedef std::map<GLenum, GLDisplayList *> Interiors;
  typedef std::map<StrokeStyle, GLDisplayList *> Strokes;

  /** Cached interior tesselations stored as a mapping from winding rule to
   *  the tesselation. */
  Interiors interiors;

  /** Cached stroke tesselations stored as a mapping from stroke style to the
   *  tesselation. */
  Strokes strokes;

 public:
  CachedTesselation();

  bool
    insert(GLenum winding_rule,
	   GLDisplayList *dl);

  bool
    insert(const StrokeStyle &style,
	   GLDisplayList *dl);

  bool
    render(GLenum winding_rule) const;

  bool
    render(const StrokeStyle &style) const;

  void
    clearInteriors();

  void
    clearStrokes();

  void
    clear();
};



#endif /* CACHES_H */
