/** colour.cc
 */
/*
    Smoke Vector Graphics engine
    Copyright (C) 1998-2003  Jon Harrop

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "colour.h"



Colour::Colour()
  : colour(convertFloatsToInt(1, 1, 1, 1))
{
}

Colour::Colour(const Num r,
	       const Num g,
	       const Num b,
	       const Num a)
  : colour(convertFloatsToInt(r, g, b, a))
{
}

unsigned int
Colour::convertFloatsToInt(const Num r,
			   const Num g,
			   const Num b,
			   const Num a) const
{
  const unsigned long
    cr = (unsigned long)(r*255),
    cg = (unsigned long)(g*255),
    cb = (unsigned long)(b*255),
    ca = (unsigned long)(a*255);

  return cr + (cg<<8) + (cb<<16) + (ca<<24);
}

Num
Colour::getRed() const
{
  const unsigned long
    c = colour & 0xff;

  return Num(c)/255;
}

Num
Colour::getGreen() const
{
  const unsigned long
    c = (colour >> 8) & 0xff;

  return Num(c)/255;
}

Num
Colour::getBlue() const
{
  const unsigned long
    c = (colour >> 16) & 0xff;

  return Num(c)/255;
}

Num
Colour::getAlpha() const
{
  const unsigned long
    c = (colour >> 24) & 0xff;

  return Num(c)/255;
}

unsigned int
Colour::getUInt() const
{
  return colour;
}

bool
Colour::operator==(const Colour c) const
{
  return colour == c.colour;
}

bool
Colour::operator!=(const Colour c) const
{
  return !(*this == c);
}

std::istream &
operator>>(std::istream &in, Colour &c)
{
  double r, g, b, a;
  in >> r >> g >> b >> a;
  c=Colour(r, g, b, a);
  return in;
}

std::ostream &
operator<<(std::ostream &out, const Colour &c)
{
  double r=c.getRed(), g=c.getGreen(), b=c.getBlue(), a=c.getAlpha();
  return out << r << " " << g << " " << b << " " << a;
}
