/** colour.h
 */
/*
    Smoke Vector Graphics engine
    Copyright (C) 1998-2003  Jon Harrop

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#ifndef COLOUR_H
#define COLOUR_H

#include "num.h"

#include <fstream>



/** Representation of colour. */
class Colour
{
 private:
  unsigned int colour;

  /** Convert three floating point numbers to one 32-bit integer. */
  unsigned int
  convertFloatsToInt(const Num r,
		     const Num g,
		     const Num b,
		     const Num a) const;

 public:
  Colour();

  Colour(const Num r,
	 const Num g,
	 const Num b,
	 const Num a);

  /** Get the proportion of RGBA red as a floating point number \f$ [0,1) \f$.
   */
  Num
    getRed() const;

  /** Get the proportion of RGBA green as a floating point number \f$ [0,1)
   * \f$. */
  Num
    getGreen() const;

  /** Get the proportion of RGBA blue as a floating point number \f$ [0,1) \f$.
   */
  Num
    getBlue() const;

  /** Get the proportion of RGBA alpha as a floating point number \f$ [0,1)
   *  \f$. */
  Num
    getAlpha() const;

  unsigned int
    getUInt() const;

  bool
    operator==(const Colour c) const;

  bool
    operator!=(const Colour c) const;
};

std::istream &
operator>>(std::istream &in, Colour &c);

std::ostream &
operator<<(std::ostream &in, const Colour &c);



#endif /* COLOUR_H */
