/** file: stroke.h
 */
/*
    Smoke Vector Graphics engine
    Copyright (C) 1998-2003  Jon Harrop

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

class LineStyle;
class StrokeStyle;

#ifndef STROKE_H
#define STROKE_H

class Store;

#include "num.h"
#include "vecmat.h"
#include "fill_types.h"

#include <list>



/** Encapsulation of all the information needed to render the geometry of any
 *  contour. */
class StrokeStyle
{
  friend class ContourGeometry;

 public:
  class Join
    {
    public:
      virtual
	void
	render(Num width,
	       ViewPort viewport,
	       const Vec &r1,
	       const Vec &r2,
	       const Vec &r3) const = 0;
    };

  class MitreJoin : public Join
    {
    public:
      void
	render(Num width,
	       ViewPort viewport,
	       const Vec &r1,
	       const Vec &r2,
	       const Vec &r3) const;
    };
  
  class BevelJoin : public Join
    {
    public:
      void
	render(Num width,
	       ViewPort viewport,
	       const Vec &r1,
	       const Vec &r2,
	       const Vec &r3) const;
    };
  
  class RoundJoin : public Join
    {
    public:
      void
	render(Num width,
	       ViewPort viewport,
	       const Vec &r1,
	       const Vec &r2,
	       const Vec &r3) const;
    };
  
  class EndCap
    {
    public:
      virtual
	void
	render(Num width,
	       ViewPort viewport,
	       const Vec &r1,
	       const Vec &r2) const;
    };
  
  class ButtEndCap : public EndCap
    {
    public:
      void
	render(Num width,
	       ViewPort viewport,
	       const Vec &r1,
	       const Vec &r2) const;
    };
  
  class RoundEndCap : public EndCap
    {
    public:
      void
	render(Num width,
	       ViewPort viewport,
	       const Vec &r1,
	       const Vec &r2) const;
    };

 private:
  /** Line width. */
  Num width;

  const Join &join;

  const EndCap &endcap0, &endcap1;

 public:
  StrokeStyle(Num w,
	      Join &j,
	      EndCap &ec1,
	      EndCap &ec2);

  void
    renderSegment(ViewPort viewport,
		  const Vec &r1,
		  const Vec &r2) const;

  void
    renderJoin(ViewPort viewport,
	       const Vec &r1,
	       const Vec &r2,
	       const Vec &r3) const;

  void
    renderEndCap(ViewPort viewport,
		 int cap,
		 const Vec &r1,
		 const Vec &r2) const;

  const Num &
    getWidth() const;

  bool
    operator<(const StrokeStyle &s) const;
};



std::ostream &
operator<<(std::ostream &out,
	   const StrokeStyle &s);



/** Style describing how to stroke and fill the strokes of any contour. */
class LineStyle
{
 public:
  typedef std::list<FillType *> FillTypes;

  LineStyle(StrokeStyle);

  FillTypes fill_types;

  StrokeStyle stroke_style;

  void
    save(std::ostream &out,
	 const Store *store) const;
};



#endif /* STROKE_H */
