/** viewport.cc
 */
/*
    Smoke Vector Graphics engine
    Copyright (C) 1998-2003  Jon Harrop

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

#include "viewport.h"

#include <cmath>



// ViewPort

ViewPort::ViewPort(const Bound b,
		   const int width,
		   const int height)
  : bound(b),
    delta(.35*(bound.u(0)-bound.l(0))/Num(width),
	  .35*(bound.u(1)-bound.l(1))/Num(height))
{
}

ViewPort::ViewPort(const Bound b,
		   const Vec d)
  : bound(b),
    delta(d)
{
}

const Bound &
ViewPort::getBound() const
{
  return bound;
}

Vec
ViewPort::getDelta() const
{
  return delta;
}



/** Transform a viewport. */
ViewPort
operator*(Mat m,
	  ViewPort vp)
{
  Vec
    d=vp.getDelta(),
    nd(std::sqrt(m(0, 0)*m(0, 0)*d(0)*d(0) + m(0, 1)*m(0, 1)*d(1)*d(1)),
       std::sqrt(m(1, 0)*m(1, 0)*d(0)*d(0) + m(1, 1)*m(1, 1)*d(1)*d(1)));

  return ViewPort(m*vp.getBound(), nd);
}
