/** viewport.h
 */
/*
    Smoke Vector Graphics engine
    Copyright (C) 1998-2003  Jon Harrop

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
*/

class ViewPort;

#ifndef VIEWPORT_H
#define VIEWPORT_H

#include "num.h"
#include "vecmat.h"
#include "bound.h"



/** Representation of an axis-aligned rectangular view of the canvas.
 *
 *  Rendering effort is concentrated on the viewable portion of the canvas
 *  represented by one of these classes and is accurate for the given
 *  sampling resolution.
 */
class ViewPort
{
 private:
  Bound bound;

  /** Delta vector. */
  Vec delta;

 public:
  /** Construct a viewport from the given bound and pixel dimensions. */
  ViewPort(const Bound b,
	   const int width,
	   const int height);

  /** Construct a viewport from the given bound and delta. */
  ViewPort(const Bound b,
	   const Vec d);

  const Bound &
    getBound() const;

  Vec
    getDelta() const;
};



/** Transform a viewport. */
ViewPort
operator*(Mat m,
	  ViewPort vp);



#endif /* VIEWPORT_H */
